<?php

declare(strict_types=1);

namespace Drupal\notifier_chat_channel;

use Symfony\Component\DependencyInjection\ChildDefinition;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Notifier\Bridge;

/**
 * Notifier Chat Channel compiler pass.
 */
final class NotifierChatChannelCompilerPass implements CompilerPassInterface {

  public function process(ContainerBuilder $container): void {
    $chatterFactories = [
      // @phpcs:disable
      'bluesky' => Bridge\Bluesky\BlueskyTransportFactory::class,
      'chatwork' => Bridge\Chatwork\ChatworkTransportFactory::class,
      'discord' => Bridge\Discord\DiscordTransportFactory::class,
      'fake-chat' => Bridge\FakeChat\FakeChatTransportFactory::class,
      'firebase' => Bridge\Firebase\FirebaseTransportFactory::class,
      'google-chat' => Bridge\GoogleChat\GoogleChatTransportFactory::class,
      'line-bot' => Bridge\LineBot\LineBotTransportFactory::class,
      'line-notify' => Bridge\LineNotify\LineNotifyTransportFactory::class,
      'linked-in' => Bridge\LinkedIn\LinkedInTransportFactory::class,
      'mastodon' => Bridge\Mastodon\MastodonTransportFactory::class,
      'mattermost' => Bridge\Mattermost\MattermostTransportFactory::class,
      'mercure' => Bridge\Mercure\MercureTransportFactory::class,
      'microsoft-teams' => Bridge\MicrosoftTeams\MicrosoftTeamsTransportFactory::class,
      'pager-duty' => Bridge\PagerDuty\PagerDutyTransportFactory::class,
      'rocket-chat' => Bridge\RocketChat\RocketChatTransportFactory::class,
      'slack' => Bridge\Slack\SlackTransportFactory::class,
      'telegram' => Bridge\Telegram\TelegramTransportFactory::class,
      'twitter' => Bridge\Twitter\TwitterTransportFactory::class,
      'zendesk' => Bridge\Zendesk\ZendeskTransportFactory::class,
      'zulip' => Bridge\Zulip\ZulipTransportFactory::class,
      // @phpcs:enable
    ];

    $parentPackages = ['symfony/framework-bundle', 'symfony/notifier'];

    foreach ($chatterFactories as $name => $class) {
      if (FALSE === ContainerBuilder::willBeAvailable(\sprintf('symfony/%s-notifier', $name), $class, $parentPackages)) {
        continue;
      }

      $factory = (new ChildDefinition('notifier.transport_factory.abstract'))
        ->setClass($class)
        ->addTag('chatter.transport_factory')
        ->setAutowired(TRUE)
        ->setAutoconfigured(TRUE);

      $container->setDefinition(\sprintf('notifier.transport_factory.%s', $name), $factory);
    }
  }

}
